/*
* The following use Javascript promise (https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Using_promises) together with the async/await keywords. Understanding promises and uses of the async/await keywords:https://www.geeksforgeeks.org/difference-between-promise-and-async-await-in-node-js/
* We use the Fetch API (https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API) to talk to the server
* Note: We could also use the common "fat arrow" (https://www.freecodecamp.org/news/when-and-why-you-should-use-es6-arrow-functions-and-when-you-shouldnt-3d851d7f0b26/) notation for our anonymous functions, but it makes the code less readable.
*/

//globals
let todoInput = document.querySelector('input[type="text"]')
let addTodoButton = document.querySelector('button')
let todoList = document.querySelector("ul")

//fetch the todos for the initial load
fetchTodos()

//click handler for the todo button
addTodoButton.onclick = addNewTodo

/**
 * post the todo to the server and adds the returning object to the DOM
 * @param {Event} evt click event
 */
async function completeTodo(evt){
    //testing if the event is on the button
    if(evt.target.tagName === "BUTTON"){
        let id = evt.target.parentNode.dataset.id
        try {
            await deleteTodoToServer(id)
            this.parentNode.removeChild(this)
        } catch(err) {
            console.log(err)
        }
    }
}

/**
 * post the todo to the server and adds the returning object to the DOM
 * @param {Event} evt button event
 */
async function addNewTodo(evt){
    let todo = todoInput.value
    if(todo.trim() != ''){
        try {
            let todoReturned = await postTodoToServer({todo:todo})
            appendTodoToList(todoReturned)
            todoInput.value = ''
        } catch(err) {
            console.log(err)
        }
    }
}


/**
 * Appends a specific json todo object to the DOM
 * @param {Object} todoJson Todo object to append
 */
function appendTodoToList(todoJson){
    let el = document.createElement("LI")
    el.innerHTML = todoJson.todo + '<button>Done</button>'
    el.dataset.id = todoJson.id
    el.onclick = completeTodo //setting the event listener for each todo
    todoList.appendChild(el)
    todoInput.value = ''
}

/**
 * Fetch todos fetch out todolist and append them to the DOM elemeent
 * async/await keywords allow us to wait for the return as object (and not promise)
 */ 
async function fetchTodos(){
    try {
        let todos = await getTodosFromServer()
        todos.forEach(todo => { appendTodoToList(todo)})
    } catch(err) {
        console.log(err)
    }
}

/**
 * Get Todo's from the server using the Fetch API
 * @return {Promise} returns a promise we have to await for
 * @return {resolved} when promise is resolve, it returns a json list of todo
 * @throws {Error} returns Error object with status code
 * 
 * The async keyword allow us to use the await prefix for the fetch function.
 */ 
 async function getTodosFromServer(){
    const response = await fetch('/todo', {
        method: "GET",
        headers: {
            'Accept': 'application/json'
        }
    })
    //We add this additional fail condition to capture connection issues
    if (!response.ok) {
        throw new Error(`Connection failed with status code ${response.status}`)
    }
    return await response.json()
}

/**
 * post todo to server
 * @return {Promise} returns a promise we have to await for
 * @return {resolved} when promise is resolve, it returns a json object with id and todo
 * @throws {Error} returns Error object with status code
 * 
 * The async keyword allow us to use the await prefix for the fetch function.
 */ 
 async function postTodoToServer(data = {}){
    const response = await fetch('/todo', {
        method: "POST",
        headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        },
        body: JSON.stringify(data)
    })
    //We add this additional fail condition to capture connection issues
    if (!response.ok) {
        throw new Error(`Connection failed with status code ${response.status}`)
    }
    return await response.json()
}

/**
 * delete todo on server
 * @return {Promise} returns a promise we have to await for
 * @return {resolved} when promise is resolve, it returns a json object with id and todo
 * @throws {Error} returns Error object with status code
 * 
 * The async keyword allow us to use the await prefix for the fetch function.
 */ 
 async function deleteTodoToServer(id){
    const response = await fetch('/todo/'+id, {
        method: "DELETE",
        headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        },
    })
    //We add this additional fail condition to capture connection issues
    if (!response.ok) {
        throw new Error(`Connection failed with status code ${response.status}`)
    }
    return
}